/**
 * $RCSfile: CMS.java,v $
 * version $Revision: 1.6 $
 * created 15.08.2007 11:42:12 by kunina
 * last modified $Date: 2009/04/22 12:52:44 $ by $Author: kunina $
 * (C)  - 2004-2007.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package CMS_samples;

import com.objsys.asn1j.runtime.Asn1BerDecodeBuffer;
import com.objsys.asn1j.runtime.Asn1BerEncodeBuffer;
import com.objsys.asn1j.runtime.Asn1Null;
import com.objsys.asn1j.runtime.Asn1ObjectIdentifier;
import com.objsys.asn1j.runtime.Asn1OctetString;
import ru.CryptoPro.JCP.ASN.CryptographicMessageSyntax.*;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.CertificateSerialNumber;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Name;
import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.params.OID;
import ru.CryptoPro.JCP.tools.Array;

import java.security.PrivateKey;
import java.security.Signature;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Arrays;


/**
 * CMS sign and verify
 * <p/>
 * csptest -sfsign -in data.txt -my key -sign -out data.sgn
 * <p/>
 * csptest -sfsign -my key -verify -in data.sgn
 * <p/>
 * csptest -lowsign -in data.txt -my key -sign -out data_low.sgn
 * <p/>
 * csptest -lowsign -in data.sgn -verify ???
 *
 * @author Copyright 2004-2009 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class CMS {
private static final String CMS_FILE = "cms_data_sgn";
private static final String CMS_FILE_PATH =
        CMStools.TEST_PATH + CMStools.SEPAR + CMS_FILE + CMStools.CMS_EXT;
/**
 * detached
 */
private static final String CMS_FILE_D = "cms_data_d_sgn";
private static final String CMS_FILE_D_PATH =
        CMStools.TEST_PATH + CMStools.SEPAR + CMS_FILE_D + CMStools.CMS_EXT;

 /**/
private CMS() {
}

/**
 * main Sign / Verify
 *
 * @param args //
 * @throws Exception e
 */
public static void main(String[] args) throws Exception {
    //    
    //CMStools.main(args);

    // prepare to sign
    final byte[] data = Array.readFile(CMStools.DATA_FILE_PATH);
    final PrivateKey key = CMStools.loadKey(CMStools.SIGN_KEY_NAME,
            CMStools.SIGN_KEY_PASSWORD);
    final Certificate cert = CMStools.loadCertificate(CMStools.SIGN_KEY_NAME);
    //final Certificate cert = CMStools.readCertificate(CMStools.SIGN_CERT_PATH);

    //Sign
    Array.writeFile(CMS_FILE_PATH, CMSSign(data, key, cert, false));
    //Verify
    CMSVerify(Array.readFile(CMS_FILE_PATH), cert, null);

    //-DETACHED-
    //Sign
    Array.writeFile(CMS_FILE_D_PATH, CMSSign(data, key, cert, true));
    //Verify
    CMSVerify(Array.readFile(CMS_FILE_D_PATH), cert, data);
}

/**
 * sign CMS
 *
 * @param data data
 * @param key key
 * @param cert cert
 * @throws Exception e
 */
public static byte[] CMSSign(byte[] data, PrivateKey key, Certificate cert,
                             boolean detached)
        throws Exception {
    // sign
    final Signature signature =
            Signature.getInstance(JCP.GOST_EL_SIGN_NAME);
    signature.initSign(key);
    signature.update(data);
    final byte[] sign = signature.sign();
    // create cms format
    return createCMS(data, sign, cert, detached);
}

/**
 * createCMS
 *
 * @param buffer buffer
 * @param sign sign
 * @param cert cert
 * @return byte[]
 * @throws Exception e
 */
public static byte[] createCMS(byte[] buffer, byte[] sign, Certificate cert,
                               boolean detached)
        throws Exception {
    final ContentInfo all = new ContentInfo();
    all.contentType = new Asn1ObjectIdentifier(
            new OID(CMStools.STR_CMS_OID_SIGNED).value);
    final SignedData cms = new SignedData();
    all.content = cms;
    cms.version = new CMSVersion(1);
    // digest
    cms.digestAlgorithms = new DigestAlgorithmIdentifiers(1);
    final DigestAlgorithmIdentifier a = new DigestAlgorithmIdentifier(
            new OID(CMStools.DIGEST_OID).value);
    a.parameters = new Asn1Null();
    cms.digestAlgorithms.elements[0] = a;
    if (detached)
        cms.encapContentInfo = new EncapsulatedContentInfo(
                new Asn1ObjectIdentifier(
                        new OID(CMStools.STR_CMS_OID_DATA).value), null);
    else
        cms.encapContentInfo =
                new EncapsulatedContentInfo(new Asn1ObjectIdentifier(
                        new OID(CMStools.STR_CMS_OID_DATA).value),
                        new Asn1OctetString(buffer));
    // certificate
    cms.certificates = new CertificateSet(1);
    final ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Certificate certificate =
            new ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Certificate();
    final Asn1BerDecodeBuffer decodeBuffer =
            new Asn1BerDecodeBuffer(cert.getEncoded());
    certificate.decode(decodeBuffer);
    cms.certificates.elements = new CertificateChoices[1];
    cms.certificates.elements[0] = new CertificateChoices();
    cms.certificates.elements[0].set_certificate(certificate);

    // signer info
    cms.signerInfos = new SignerInfos(1);
    cms.signerInfos.elements[0] = new SignerInfo();
    cms.signerInfos.elements[0].version = new CMSVersion(1);
    cms.signerInfos.elements[0].sid = new SignerIdentifier();

    final byte[] encodedName = ((X509Certificate) cert).getIssuerX500Principal()
            .getEncoded();
    final Asn1BerDecodeBuffer nameBuf = new Asn1BerDecodeBuffer(encodedName);
    final Name name = new Name();
    name.decode(nameBuf);

    final CertificateSerialNumber num = new CertificateSerialNumber(
            ((X509Certificate) cert).getSerialNumber());
    cms.signerInfos.elements[0].sid.set_issuerAndSerialNumber(
            new IssuerAndSerialNumber(name, num));
    cms.signerInfos.elements[0].digestAlgorithm =
            new DigestAlgorithmIdentifier(new OID(CMStools.DIGEST_OID).value);
    cms.signerInfos.elements[0].digestAlgorithm.parameters = new Asn1Null();
    cms.signerInfos.elements[0].signatureAlgorithm =
            new SignatureAlgorithmIdentifier(new OID(CMStools.SIGN_OID).value);
    cms.signerInfos.elements[0].signatureAlgorithm.parameters = new Asn1Null();
    cms.signerInfos.elements[0].signature = new SignatureValue(sign);
    // encode
    final Asn1BerEncodeBuffer asnBuf = new Asn1BerEncodeBuffer();
    all.encode(asnBuf, true);
    return asnBuf.getMsgCopy();
}

/**
 * verify CMS
 *
 * @param buffer buffer
 * @param cert cert
 * @param data data
 * @throws Exception e
 */
public static void CMSVerify(byte[] buffer, Certificate cert, byte[] data)
        throws Exception {
    int i;
    final Asn1BerDecodeBuffer asnBuf = new Asn1BerDecodeBuffer(buffer);
    final ContentInfo all = new ContentInfo();
    all.decode(asnBuf);
    if (!new OID(CMStools.STR_CMS_OID_SIGNED).eq(all.contentType.value))
        throw new Exception("Not supported");
    final SignedData cms = (SignedData) all.content;
    if (cms.version.value != 1)
        throw new Exception("Incorrect version");
    if (!new OID(CMStools.STR_CMS_OID_DATA).eq(
            cms.encapContentInfo.eContentType.value))
        throw new Exception("Nested not supported");
    byte[] text = null;
    if (data != null) text = data;
    else if (cms.encapContentInfo.eContent != null)
        text = cms.encapContentInfo.eContent.value;
    if (text == null) throw new Exception("No content");
    OID digestOid = null;
    DigestAlgorithmIdentifier a = new DigestAlgorithmIdentifier(
            new OID(CMStools.DIGEST_OID).value);
    for (i = 0; i < cms.digestAlgorithms.elements.length; i++) {
        if (cms.digestAlgorithms.elements[i].algorithm.equals(a.algorithm)) {
            digestOid =
                    new OID(cms.digestAlgorithms.elements[i].algorithm.value);
            break;
        }
    }
    if (digestOid == null)
        throw new Exception("Unknown digest");
    int pos = -1;
    for (i = 0; i < cms.certificates.elements.length; i++) {
        final Asn1BerEncodeBuffer encBuf = new Asn1BerEncodeBuffer();
        cms.certificates.elements[i].encode(encBuf);
        final byte[] in = encBuf.getMsgCopy();
        if (Arrays.equals(in, cert.getEncoded())) {
            pos = i;
            break;
        }
    }
    if (pos == -1)
        throw new Exception("Not signed on certificate.");

    final SignerInfo info = cms.signerInfos.elements[pos];
    if (info.version.value != 1)
        throw new Exception("Incorrect version");
    if (!digestOid.equals(new OID(info.digestAlgorithm.algorithm.value)))
        throw new Exception("Not signed on certificate.");
    final byte[] sign = info.signature.value;

    // check
    final Signature signature = Signature.getInstance(JCP.GOST_EL_SIGN_NAME);
    signature.initVerify(cert);
    signature.update(text);
    final boolean checkResult = signature.verify(sign);
    if (checkResult) CMStools.logger.info("Valid signature");
    else throw new Exception("Invalid signature.");
    //return text;
}
}
