/**
 * $RCSfile: CMSSignAndEncrypt.java,v $
 * version $Revision: 1.5 $
 * created 21.05.2008 12:17:43 by kunina
 * last modified $Date: 2009/04/22 12:52:44 $ by $Author: kunina $
 * (C)  - 2004-2008.
 *
 *  ,    , 
 *   .     
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package CMS_samples;

import com.objsys.asn1j.runtime.Asn1BerDecodeBuffer;
import com.objsys.asn1j.runtime.Asn1BerEncodeBuffer;
import com.objsys.asn1j.runtime.Asn1Null;
import com.objsys.asn1j.runtime.Asn1ObjectIdentifier;
import com.objsys.asn1j.runtime.Asn1OctetString;
import ru.CryptoPro.JCP.ASN.CryptographicMessageSyntax.*;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax.Gost28147_89_EncryptedKey;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax.Gost28147_89_IV;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax.Gost28147_89_ParamSet;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax.Gost28147_89_Parameters;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax._Gost28147_89_EncryptionSyntaxValues;
import ru.CryptoPro.JCP.ASN.GostR3410_EncryptionSyntax.GostR3410_KeyTransport;
import ru.CryptoPro.JCP.ASN.GostR3410_EncryptionSyntax.GostR3410_TransportParameters;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.AlgorithmIdentifier;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Certificate;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.CertificateSerialNumber;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Name;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.RDNSequence;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.SubjectPublicKeyInfo;
import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.params.AlgIdInterface;
import ru.CryptoPro.JCP.params.AlgIdSpec;
import ru.CryptoPro.JCP.params.OID;
import ru.CryptoPro.JCP.params.ParamsInterface;
import ru.CryptoPro.JCP.tools.Array;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.security.auth.x500.X500Principal;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.security.cert.X509Certificate;

/**
 * CryptSignAndEncryptMessage.
 * <p/>
 * CryptDecryptAndVerivyMessage with:
 * <p/>
 * csptest -sfse -decrypt -in envCMS.txt -out envCMS.txt.decr
 *
 * @author Copyright 2004-2009 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class CMSSignAndEncrypt {

private static final String CMS_FILE = "cms_data_encr_sgn";
protected static final String CMS_FILE_PATH =
        CMStools.TEST_PATH + CMStools.SEPAR + CMS_FILE;
protected static final String CIPHER_MODE = "GOST28147/CFB/NoPadding";

/**
 *    
 */
private static final byte[] sv =
        {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11};

public static void main(String[] args) throws Exception {
    //      
    final byte[] data = Array.readFile(CMStools.DATA_FILE_PATH);

    // 
    final KeyStore hdImageStore = KeyStore.getInstance(CMStools.STORE_TYPE);
    hdImageStore.load(null, null);
    // 
    final PrivateKey senderKey =
            (PrivateKey) hdImageStore
                    .getKey(CMStools.SIGN_KEY_NAME, CMStools.SIGN_KEY_PASSWORD);
    final X509Certificate publicSenderCert =
            (X509Certificate) hdImageStore
                    .getCertificate(CMStools.SIGN_KEY_NAME);
    // 
    final X509Certificate publicCert =
            (X509Certificate) hdImageStore
                    .getCertificate(CMStools.RECIP_KEY_NAME);

    // SignedData
    final ContentInfo contentSign = new ContentInfo();
    contentSign.contentType =
            new Asn1ObjectIdentifier(
                    new OID(CMStools.STR_CMS_OID_SIGNED).value);
    final SignedData signedData = new SignedData();
    contentSign.content = signedData;
    signedData.version = new CMSVersion(1);
    // digest
    signedData.digestAlgorithms = new DigestAlgorithmIdentifiers(1);
    final DigestAlgorithmIdentifier a = new DigestAlgorithmIdentifier(
            new OID(CMStools.DIGEST_OID).value);
    a.parameters = new Asn1Null();
    signedData.digestAlgorithms.elements[0] = a;
    signedData.encapContentInfo =
            new EncapsulatedContentInfo(new Asn1ObjectIdentifier(
                    new OID(CMStools.STR_CMS_OID_DATA).value),
                    new Asn1OctetString(data));
    // certificates
    signedData.certificates = new CertificateSet(1);
    signedData.certificates.elements = new CertificateChoices[1];
    final Certificate certificate = new Certificate();
    final Asn1BerDecodeBuffer decodeBuffer =
            new Asn1BerDecodeBuffer(publicSenderCert.getEncoded());
    certificate.decode(decodeBuffer);
    signedData.certificates.elements[0] = new CertificateChoices();
    signedData.certificates.elements[0].set_certificate(certificate);
    // Signature.getInstance
    final Signature signature = Signature.getInstance(JCP.GOST_EL_SIGN_NAME);
    // signer infos
    signedData.signerInfos = new SignerInfos(1);
    signature.initSign(senderKey);
    signature.update(data);
    final byte[] sign = signature.sign();
    signedData.signerInfos.elements[0] = new SignerInfo();
    signedData.signerInfos.elements[0].version = new CMSVersion(1);
    signedData.signerInfos.elements[0].sid = new SignerIdentifier();

    final byte[] encodedName =
            publicSenderCert.getIssuerX500Principal()
                    .getEncoded();
    final Asn1BerDecodeBuffer nameBuf =
            new Asn1BerDecodeBuffer(encodedName);
    final Name name = new Name();
    name.decode(nameBuf);

    final CertificateSerialNumber num = new CertificateSerialNumber(
            publicSenderCert.getSerialNumber());
    signedData.signerInfos.elements[0].sid.set_issuerAndSerialNumber(
            new IssuerAndSerialNumber(name, num));
    signedData.signerInfos.elements[0].digestAlgorithm =
            new DigestAlgorithmIdentifier(new OID(CMStools.DIGEST_OID).value);
    signedData.signerInfos.elements[0].digestAlgorithm.parameters =
            new Asn1Null();
    signedData.signerInfos.elements[0].signatureAlgorithm =
            new SignatureAlgorithmIdentifier(
                    new OID(CMStools.SIGN_OID).value);
    signedData.signerInfos.elements[0].signatureAlgorithm.parameters =
            new Asn1Null();
    signedData.signerInfos.elements[0].signature = new SignatureValue(sign);

    // encode
    final Asn1BerEncodeBuffer asnBuf = new Asn1BerEncodeBuffer();
    contentSign.encode(asnBuf, true);

    //  envelopedData
    final byte[] buffer = asnBuf.getMsgCopy();

    //===============================

    final PublicKey responderPublic = publicCert.getPublicKey();

    //    
    final KeyAgreement senderKeyAgree =
            KeyAgreement.getInstance(CMStools.KEY_ALG_NAME);
    senderKeyAgree.init(senderKey, new IvParameterSpec(sv), null);
    senderKeyAgree.doPhase(responderPublic, true);
    final SecretKey alisaSecret =
            senderKeyAgree.generateSecret(CMStools.SEC_KEY_ALG_NAME);

    //         .
    final KeyGenerator kg = KeyGenerator.getInstance(CMStools.SEC_KEY_ALG_NAME);
    final ParamsInterface paramss = AlgIdSpec.getDefaultCryptParams();
    kg.init(paramss);
    final SecretKey simm = kg.generateKey();

    //     .
    final Cipher cipher = Cipher.getInstance(CIPHER_MODE);
    cipher.init(Cipher.ENCRYPT_MODE, simm, (SecureRandom) null);
    final byte[] iv = cipher.getIV();
    final byte[] text = cipher.doFinal(buffer, 0, buffer.length);

    //      
    cipher.init(Cipher.WRAP_MODE, alisaSecret, (SecureRandom) null);
    final byte[] key = cipher.wrap(simm);

    // CMS-
    final ContentInfo all = new ContentInfo();
    all.contentType =
            new Asn1ObjectIdentifier(
                    new OID(CMStools.STR_CMS_OID_ENVELOPED).value);
    final EnvelopedData cms = new EnvelopedData();
    all.content = cms;

    cms.version = new CMSVersion(0);

    cms.recipientInfos = new RecipientInfos(1);
    cms.recipientInfos.elements = new RecipientInfo[1];
    cms.recipientInfos.elements[0] = new RecipientInfo();

    final KeyTransRecipientInfo keytrans = new KeyTransRecipientInfo();

    keytrans.version = new CMSVersion(0);

    final Asn1BerEncodeBuffer ebuf = new Asn1BerEncodeBuffer();
    final SubjectPublicKeyInfo spki = new SubjectPublicKeyInfo();
    final Asn1BerDecodeBuffer dbuff = new Asn1BerDecodeBuffer(
            publicSenderCert.getPublicKey().getEncoded());
    spki.decode(dbuff);
    dbuff.reset();
    final AlgIdInterface algid = new AlgIdSpec(spki.algorithm);
    final AlgorithmIdentifier id = (AlgorithmIdentifier) algid.getDecoded();
    id.encode(ebuf);
    Asn1BerDecodeBuffer dbuf = new Asn1BerDecodeBuffer(ebuf.getMsgCopy());
    keytrans.keyEncryptionAlgorithm = new KeyEncryptionAlgorithmIdentifier();
    keytrans.keyEncryptionAlgorithm.decode(dbuf);
    ebuf.reset();
    dbuf.reset();
    keytrans.rid = new RecipientIdentifier();
    final IssuerAndSerialNumber issuer = new IssuerAndSerialNumber();
    final X500Principal issuerName = publicCert.getIssuerX500Principal();
    dbuf = new Asn1BerDecodeBuffer(issuerName.getEncoded());
    issuer.issuer = new Name();
    final RDNSequence rnd = new RDNSequence();
    rnd.decode(dbuf);
    issuer.issuer.set_rdnSequence(rnd);
    issuer.serialNumber = new CertificateSerialNumber(
            publicCert.getSerialNumber());
    keytrans.rid.set_issuerAndSerialNumber(issuer);
    dbuf.reset();
    final GostR3410_KeyTransport encrKey = new GostR3410_KeyTransport();
    dbuf = new Asn1BerDecodeBuffer(key);
    encrKey.sessionEncryptedKey = new Gost28147_89_EncryptedKey();
    encrKey.sessionEncryptedKey.decode(dbuf);
    dbuf.reset();
    encrKey.transportParameters = new GostR3410_TransportParameters();
    encrKey.transportParameters.encryptionParamSet =
            new Gost28147_89_ParamSet(algid.getCryptParams()
                    .getOID().value);
    encrKey.transportParameters.ephemeralPublicKey = new SubjectPublicKeyInfo();
    dbuf = new Asn1BerDecodeBuffer(
            publicSenderCert.getPublicKey().getEncoded());
    encrKey.transportParameters.ephemeralPublicKey.decode(dbuf);
    dbuf.reset();
    encrKey.transportParameters.ukm = new Asn1OctetString(sv);
    encrKey.encode(ebuf);
    keytrans.encryptedKey = new EncryptedKey(ebuf.getMsgCopy());
    ebuf.reset();
    cms.recipientInfos.elements[0].set_ktri(keytrans);

    cms.encryptedContentInfo = new EncryptedContentInfo();
    final OID contentType = new OID(CMStools.STR_CMS_OID_DATA);
    cms.encryptedContentInfo.contentType = new ContentType(contentType.value);
    final Gost28147_89_Parameters params = new Gost28147_89_Parameters();
    params.iv = new Gost28147_89_IV(iv);
    params.encryptionParamSet =
            new Gost28147_89_ParamSet(paramss.getOID().value);
    cms.encryptedContentInfo.contentEncryptionAlgorithm =
            new ContentEncryptionAlgorithmIdentifier(
                    _Gost28147_89_EncryptionSyntaxValues.id_Gost28147_89,
                    params);
    cms.encryptedContentInfo.encryptedContent = new EncryptedContent(text);

    all.encode(ebuf);
    Array.writeFile(CMS_FILE_PATH, ebuf.getMsgCopy());
    CMStools.logger.info("ok");
}
}
