/**
 * $RCSfile: CheckConfFull.java,v $
 * version $Revision: 1.3 $
 * created 30.09.2008 10:37:34 by kunina
 * last modified $Date: 2009/04/21 11:44:18 $ by $Author: kunina $
 * (C)  - 2004-2008.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package ComLine;

import JTLS_samples.Client;
import JTLS_samples.Server;
import ru.CryptoPro.JCP.tools.Array;
import ru.CryptoPro.JCPRequest.GostCertificateRequest;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.util.Properties;
import java.util.logging.Logger;

/**
 *   .
 *
 * @author Copyright 2004-2008 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class CheckConfFull {

private static boolean f_jcp = false;
private static boolean f_crypto = false;
private static boolean f_jtls = false;
private static Logger log = Logger.getLogger("Log");
private static String signKey = "jcptestsignkey";
private static String kpAlg = "GOST3410";
private static String kpAlgDH = "GOST3410DH";
private static String ksName = "HDImageStore";
private static char[] keyPass = null;
private static char[] keyPass_1 = "jcptest".toCharArray();
private static String dNameCN = "CN=";
private static String dNameE = ",OU=Security,O=CryptoPro,C=RU";
private static String signAlg = "GOST3411withGOST3410EL";
private static String dhKey = "jcptestcryptokey";
private static int sslPort = 1212;
private static String serverKey = "jcptestserverkey";
private static String clientKey = "jcptestclientkey";
private static String trust = "jcptesttrust.store";
private static String in = "jcptestdoc.in";
private static String out = "jcptestdoc.out";
private static String out_a = "jcptestdoc_a.out";
private static String doc;
private static String get_doc;
private static String get_doc_a;
private static String trustSt;
private static String clientKeyCert = clientKey + "cert";

 /**/
private CheckConfFull() {
}

/**
 * CheckConfFull [-servDir C:/*.*]
 * <p/>
 * <DL> <DT><b> -servDir </b>  <DD> <DD>( 
 * )</DD></DT> </DL>
 *
 * @param args   
 * @throws IOException /
 */
public static void main(String[] args) throws IOException {
    if (ComLine.getFunc(ComLine.hllp, args))
        log.info(ComLine.CheckConfFullHelp);
    else {
        checkProvs();
        if (f_jcp) {
            if (checkJCP()) log.info("\nJCP - OK\n");
            else log.warning("\nJCP - FAIL\n");
            if (f_crypto) {
                if (checkCrypto()) log.info("\nCrypto - OK\n");
                else log.warning("\nCrypto - FAIL\n");
                if (f_jtls) {
                    final Properties ArgList = new Properties();
                    //server working dir
                    ArgList.setProperty(ComLine.servDir,
                            ComLine.getValue(ComLine.servDir,
                                    args, new File(".").getCanonicalPath()));

                    if (checkJTLS(ArgList)) log.info("\nJTLS - OK\n");
                    else log.warning("\nJTLS - FAIL\n");
                }
            }
        }
    }
}

/**
 *   
 */
private static void checkProvs() {
    final Provider[] provs = Security.getProviders();
    for (int i = 0; i < provs.length; i++) {
        if ("JCP".equals(provs[i].getName()))
            f_jcp = true;
        if ("Crypto".equals(provs[i].getName()))
            f_crypto = true;
        if ("JTLS".equals(provs[i].getName()))
            f_jtls = true;
    }
    if (!f_jcp) log.info("Provider JCP not installed.");
    if (!f_crypto) log.info("Provider Crypto not installed.");
    if (!f_jtls) log.info("Provider JTLS  not installed.");
}

 /**/
private static boolean checkJCP() {
    boolean ok = true;
    try {
        log.info("***Check JCP key pair generation and writing to store");
        final String dName = dNameCN + signKey + dNameE;
        if (keyGen(signKey, kpAlg, keyPass, dName, false)) log.info(
                "***Check JCP key pair generation and writing to store - OK");
    } catch (Exception e) {
        ok = false;
        e.printStackTrace();
    }
    try {
        log.info("***Check JCP signature generation|verifycation");
        if (signVer(signKey, keyPass))
            log.info("***JCP signature generation|verifycation - OK");
    } catch (Exception e) {
        ok = false;
        e.printStackTrace();
    }
    return ok;
}

 /**/
private static boolean checkCrypto() {
    boolean ok = true;
    try {
        log.info("***Check Crypto key pair generation and writing to store");
        final String dName = dNameCN + dhKey + dNameE;
        if (keyGen(dhKey, kpAlgDH, keyPass, dName, false)) log.info(
                "***Check Crypto key pair generation and writing to store - OK");
    } catch (Exception e) {
        ok = false;
        e.printStackTrace();
    }
    return ok;
}

 /**/
private static boolean checkJTLS(Properties argList) {
    boolean ok = true;
    try {
        log.info("***Prepare keys");
        String dName = dNameCN + serverKey + dNameE;
        keyGen(serverKey, kpAlgDH, keyPass_1, dName, true);
        dName = dNameCN + clientKey + dNameE;
        keyGen(clientKey, kpAlgDH, keyPass_1, dName, false);
        log.info("***Prepare files");
        final String dir = argList.getProperty(ComLine.servDir);
        doc = dir + File.separator + in;
        get_doc = dir + File.separator + out;
        get_doc_a = dir + File.separator + out_a;
        Array.writeFile(doc, "12345".getBytes());
        if (!new File(doc).exists())
            throw new Exception("Can't create file for test");
        log.info("***Prepare TrustStore");
        trustSt = dir + File.separator + trust;
        createTrustStore(trustSt, clientKey, clientKeyCert);
        log.info("***Test sll without autentification of client");
        tls(false, dir, get_doc);
        log.info("***Test sll with autentification of client");
        tls(false, dir, get_doc_a);

    } catch (Exception e) {
        ok = false;
        e.printStackTrace();
    }

    return ok;
}

/**
 *    
 */
private static void createTrustStore(String stPath, String alias, String tAlias)
        throws Exception {
    //   
    final KeyStore ks = KeyStore.getInstance(ksName);
    ks.load(null, null);
    final Certificate cert = ks.getCertificate(alias);
    if (cert == null)
        throw new Exception("Certificate named \"" + alias + "\" not found");
    log.info("Loading of a certificate is completed");
    ks.setCertificateEntry(tAlias, cert);
    ks.store(new FileOutputStream(stPath), keyPass_1);


}

/**
 * TLS 
 */
private static boolean tls(boolean isAuth, String dir, String outPath)
        throws Exception {
    boolean ok = true;
    Server server = null;
    try {
        //  
        System.setProperty("javax.net.ssl.keyStoreType", ksName);
        System.setProperty("javax.net.ssl.keyStorePassword",
                new String(keyPass_1));
        //  
        System.setProperty("javax.net.ssl.trustStoreType", ksName);
        System.setProperty("javax.net.ssl.trustStore", trustSt);
        System.setProperty("javax.net.ssl.trustStorePassword",
                new String(keyPass_1));

        server = new Server(sslPort, isAuth, dir);
        server.setTimeout(100000);
        server.start();

        final Client client = new Client("localhost", sslPort);
        client.setTimeout(100000);

        if (!server.isAlive())
            throw new Exception("server not running");
        else log.info("server started");

        client.get(in, outPath, dir);

        if (!server.isAlive())
            throw new Exception("server not alive after client Get");
        else log.info("GET complete");

    } catch (Exception e) {
        ok = false;
        e.printStackTrace();
    } finally {
        if (server == null) {
            ok = false;
            throw new Exception("server not running at the test");
        }
        server.stop();
        Thread.sleep(100);
    }
    return ok;
}

/**
 *       
 */
private static boolean keyGen(String alias, String keypairAlgorithm,
                              char[] keypass, String name,
                              boolean isServer)
        throws Exception {
    //  
    final KeyPairGenerator kg =
            KeyPairGenerator.getInstance(keypairAlgorithm);
    final KeyPair pair = kg.generateKeyPair();
    log.info("Generation of key pair is completed (" + keypairAlgorithm + ")");
    //  
    final GostCertificateRequest req;
    req = new GostCertificateRequest();
    req.init(keypairAlgorithm, isServer);
    final byte[] encodedCert = req.getEncodedSelfCert(pair, name);

    //  
    final CertificateFactory cf = CertificateFactory.getInstance("X509");
    final Certificate[] certs;
    certs = new Certificate[1];
    certs[0] =
            cf.generateCertificate(new ByteArrayInputStream(encodedCert));
    log.info("Generation of certificate is completed"); /**/

    //       
    final KeyStore ks = KeyStore.getInstance(ksName);
    ks.load(null, null);
    final PrivateKey key;
    key = pair.getPrivate();
    ks.setKeyEntry(alias, key, keypass, certs);
    ks.store(null, null);
    log.info("Recording of a private key named \"" + alias + "\" to " +
            ksName + " is completed.");
    return true;
}

/**
 *    
 */
private static boolean signVer(String alias, char[] keypass) throws Exception {
    //    HDImageStore
    final KeyStore ks = KeyStore.getInstance(ksName);
    ks.load(null, null);
    final PrivateKey privateKey;
    privateKey = (PrivateKey) ks.getKey(alias, keypass);
    if (privateKey == null)
        throw new Exception("Key named \"" + alias + "\" not found");
    log.info("Loading of a private key is completed");

    //
    final byte[] text = "sample".getBytes();
    log.info("Loading of a text is completed");

    // 
    final java.security.Signature sig;
    sig = java.security.Signature.getInstance(signAlg);
    sig.initSign(privateKey);
    sig.update(text);

    //
    final byte[] signature = sig.sign();
    Logger.getLogger("LOGGER").info("Generation of the signature is completed");

    //     HDImageStore
    ks.load(null, null);
    final Certificate cert = ks.getCertificate(alias);
    final PublicKey publicKey;
    publicKey = cert.getPublicKey();
    if (publicKey == null)
        throw new Exception("Key named \"" + alias + "\" not found");
    log.info("Loading of a public key is completed");

    // 
    final java.security.Signature sign;
    sign = java.security.Signature.getInstance(signAlg);
    sign.initVerify(publicKey);
    sign.update(text);
    final boolean verifies;
    verifies = sign.verify(signature);
    final String s;
    if (verifies) s = "The signature is true";
    else s = "The signature is not true";
    log.info(s);
    return verifies;
}
}
