/**
 * $RCSfile: Certificates.java,v $
 * version $Revision: 1.3 $
 * created 27.05.2009 18:45:02 by kunina
 * last modified $Date: 2011/03/16 16:44:47 $ by $Author: iva $
 * (C)  - 2004-2009.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package userSamples;

import com.objsys.asn1j.runtime.Asn1Boolean;
import com.objsys.asn1j.runtime.Asn1ObjectIdentifier;
import com.objsys.asn1j.runtime.Asn1OctetString;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.Extension;
import ru.CryptoPro.JCPRequest.GostCertificateRequest;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.cert.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *    .
 *
 * @author Copyright 2004-2009 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class Certificates {
/**
 *    
 */
private static final String ALIAS = "newCert";
/**
 *       
 */
private static final String DNAME = "CN=newCert, O=CryptoPro, C=RU";
/**
 * http-   
 */
private static final String HTTP_ADDRESS = "http://www.cryptopro.ru/certsrv/";
/**
 *      
 */
private static final String STORE_TYPE = Constants.KEYSTORE_TYPE;
/**
 *  
 */
private static final String KEY_ALG = Constants.SIGN_KEY_PAIR_ALG;
/**
 *     
 */
private static final char[] STORE_PASS = "password".toCharArray();
/**
 *     
 */
private static final String STORE_PATH =
        System.getProperty("user.home") + File.separator + "new.keystore";
/**
 *    
 */
private static final String CERT_PATH =
        System.getProperty("user.home") + File.separator + "newCertificate.cer";


/**
 * @param args null
 * @throws Exception /
 */
public static void main(String[] args) throws Exception {
    //      
    writeCertSample();
    //        
    readCertSample();
    //  
    // (    /)
    //certificateChain();
}

/**
 *   ,      
 *        
 *
 * @throws Exception /
 */
public static void writeCertSample() throws Exception {
    /*          
     */
    KeyPair keypair = KeyPairGen.genKey(KEY_ALG);
    //        
    //   DER-
    byte[] encoded = createRequestAndGetCert(keypair);

    //   X509-
    CertificateFactory cf = CertificateFactory.getInstance("X509");
    //  X509-    
    Certificate cert =
            cf.generateCertificate(new ByteArrayInputStream(encoded));

    /*     */
    //       
    // ( )
    KeyStore keyStore = KeyStore.getInstance(STORE_TYPE);
    //    (,  
    //   CertStoreName  , .. 
    //      )
    keyStore.load(null, null);

    //      
    // (,      CertStoreName  
    //      alias)
    keyStore.setCertificateEntry(ALIAS, cert);

    //          
    File file = new File(STORE_PATH);
    //     
    keyStore.store(new FileOutputStream(file), STORE_PASS);
}

/**
 *          
 *
 * @throws Exception /
 */
public static void readCertSample() throws Exception {
    /*       */
    //       
    // ( )
    final KeyStore keyStore = KeyStore.getInstance(STORE_TYPE);
    //        
    //    
    final File file = new File(STORE_PATH);
    //    (,  ,
    //   CertStoreName )
    keyStore.load(new FileInputStream(file), STORE_PASS);

    //      
    // (,      CertStoreName  
    //      alias)
    final Certificate cert = keyStore.getCertificate(ALIAS);

    //         
    keyStore.store(new FileOutputStream(file), STORE_PASS);

    /*      */
    //         
    final File cert_file = new File(CERT_PATH);
    //    DER-
    final byte[] encoded = cert.getEncoded();
    //     
    final FileOutputStream outStream = new FileOutputStream(cert_file);
    outStream.write(encoded);
}

/**
 *  ,     
 * ,    -  .  
 *           .
 *
 * @throws Exception /
 */
public static void certificateChain() throws Exception {
    //    
    final String aliasRootCert = "rootCert";
    //    
    final String aliasInterCert = "intermediateCert";
    //     
    final String aliasEndCert = "endCert";

    //       
    final KeyStore keyStore = KeyStore.getInstance(STORE_TYPE);

    //    (,  ,
    //   STORE_TYPE )  
    //  
    keyStore.load(new FileInputStream(STORE_PATH), STORE_PASS);

    //       
    // (,      )
    final Certificate certRoot = keyStore.getCertificate(aliasRootCert);

    //       
    // (,      )
    final Certificate certInter = keyStore.getCertificate(aliasInterCert);

    //    (  )  
    // (,       )
    final Certificate certEnd = keyStore.getCertificate(aliasEndCert);

    //         
    keyStore.store(new FileOutputStream(STORE_PATH), STORE_PASS);

    //    ,    
    //(  aliasRootCert)      (c  aliasEndCert)

    //   ,  
    //   
    final List certs = new ArrayList(3);
    certs.add(certRoot);
    certs.add(certInter);
    certs.add(certEnd);

    //    (   
    // )
    final TrustAnchor anchor =
            new TrustAnchor((X509Certificate) certRoot, null);

    //    
    // ,     
    //    
    final CollectionCertStoreParameters par =
            new CollectionCertStoreParameters(certs);

    //      
    // ,   
    final CertStore store = CertStore.getInstance("Collection", par);

    //     
    final CertPathBuilder cpb = CertPathBuilder.getInstance("PKIX");
    //     
    //CertPathBuilder cpb = CertPathBuilder.getInstance("CPPKIX");

    //     
    final PKIXBuilderParameters params = new PKIXBuilderParameters(
            Collections.singleton(anchor), new X509CertSelector());

    //    ,  
    //   
    params.addCertStore(store);

    //    , 
    //   
    final X509CertSelector selector = new X509CertSelector();

    //  , 
    //   
    selector.setCertificate((X509Certificate) certEnd);

    params.setTargetCertConstraints(selector);

    //   
    final PKIXCertPathBuilderResult res =
            (PKIXCertPathBuilderResult) cpb.build(params);

    /*     */

    //     
    final CertPathValidator validator = CertPathValidator.getInstance("PKIX");
    //     
    //CertPathValidator validator = CertPathValidator.getInstance("CPPKIX");

    //   
    final CertPathValidatorResult val_res =
            validator.validate(res.getCertPath(), params);

    //      
    System.out.println(val_res.toString());
}

/**
 *     ,    
 *     .
 *
 * @param pair  .       ,
 *     .
 * @return   DER-
 * @throws Exception errors
 */
public static byte[] createRequestAndGetCert(KeyPair pair) throws Exception {
    /*         */
    //     
    GostCertificateRequest request = new GostCertificateRequest();
    //  
    // @deprecated   1.0.48
    //  init()   setKeyUsage()  addExtKeyUsage()
    // request.init(KEY_ALG);

    /*
      keyUsage     
     setKeyUsage.     , ..    
       init()  "GOST3410"  
     DIGITAL_SIGNATURE | NON_REPUDIATION.   , .. 
      "GOST3410DH"  KEY_ENCIPHERMENT | KEY_AGREEMENT.
     */
    int keyUsage = GostCertificateRequest.DIGITAL_SIGNATURE |
            GostCertificateRequest.NON_REPUDIATION |
            GostCertificateRequest.KEY_ENCIPHERMENT |
            GostCertificateRequest.KEY_AGREEMENT;
    request.setKeyUsage(keyUsage);
    /*
      ExtendedKeyUsage  .     ,
     ..   "GOST3410"   .  
     , ..   "GOST3410DH"  OID
     INTS_PKIX_CLIENT_AUTH "1.3.6.1.5.5.7.3.2",     true
        init()   INTS_PKIX_SERVER_AUTH
     "1.3.6.1.5.5.7.3.1"
     */
    request.addExtKeyUsage(GostCertificateRequest.INTS_PKIX_EMAIL_PROTECTION);
    /**
     * ExtendedKeyUsage    "1.3.6.1.5.5.7.3.3",  
     *  int[]{1, 3, 6, 1, 5, 5, 7, 3, 4}   
     * ru.CryptoPro.JCP.params.OID
     */
    request.addExtKeyUsage("1.3.6.1.5.5.7.3.3");
    /**
     *       Basic Constraints
     */
    Extension ext = new Extension();
    int[] extOid = {2, 5, 29, 19};
    ext.extnID = new Asn1ObjectIdentifier(extOid);
    ext.critical = new Asn1Boolean(true);
    byte[] extValue = {48, 6, 1, 1, -1, 2, 1, 5};
    ext.extnValue = new Asn1OctetString(extValue);
    request.addExtension(ext);

    //      
    request.setPublicKeyInfo(pair.getPublic());
    //      
    request.setSubjectInfo(DNAME);
    //        
    request.encodeAndSign(pair.getPrivate());

    //        
    //   DER-
    return request.getEncodedCert(HTTP_ADDRESS);
}
}
