/**
 * $RCSfile: OCSPValidateCert.java,v $
 * version $Revision: 1.2 $
 * created 06.10.2008 17:31:06 by kunina
 * last modified $Date: 2008/10/07 11:35:08 $ by $Author: kunina $
 * (C)  - 2004-2008.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package userSamples;

import ru.CryptoPro.JCPRequest.GostCertificateRequest;

import java.io.ByteArrayInputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.Security;
import java.security.cert.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *   java 1.5  .
 * <p/>
 * http://java.sun.com/javase/6/docs/technotes/guides/security/certpath/CertPathProgGuide.html
 *
 * @author Copyright 2004-2008 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class OCSPValidateCert {

public static void main(String[] args) throws Exception {

    final KeyPairGenerator kg =
            KeyPairGenerator.getInstance("GOST3410");
    final KeyPair pair = kg.generateKeyPair();

    final GostCertificateRequest req = new GostCertificateRequest();
    req.init("GOST3410", false);
    req.setPublicKeyInfo(pair.getPublic());
    req.setSubjectInfo("CN=TEST_OCSP");
    req.encodeAndSign(pair.getPrivate());

    final byte[] encodedCert =
            req.getEncodedCert("http://www.cryptopro.ru/certsrv/");
    final byte[] encodedRootCert = GostCertificateRequest
            .getEncodedRootCert("http://www.cryptopro.ru/certsrv/");
    final CertificateFactory cf = CertificateFactory.getInstance("X509");
    final Certificate crt =
            cf.generateCertificate(new ByteArrayInputStream(encodedCert));
    final Certificate tr =
            cf.generateCertificate(new ByteArrayInputStream(encodedRootCert));

    // ocsp
    Security.setProperty("ocsp.enable", "true");
    //  
    // 
    Security.setProperty("ocsp.responderCertSubjectName",
            "CN=Test Center CRYPTO-PRO,O=CRYPTO-PRO,C=RU,EMAILADDRESS=info@cryptopro.ru");
    //    
    Security.setProperty("ocsp.responderURL",
            "http://www.cryptopro.ru/ocspnc/ocsp.srf");

    // (     ,  )
    final Certificate[] certs = new Certificate[2];
    certs[0] = crt;
    certs[1] = tr;  //root

    final Set trust = new HashSet(0);
    trust.add(new TrustAnchor((X509Certificate) tr, null));

    final List cert = new ArrayList(0);
    for (int i = 0; i < certs.length; i++)
        cert.add(certs[i]);

    //
    final PKIXBuilderParameters cpp = new PKIXBuilderParameters(trust, null);
    cpp.setSigProvider(null);
    final CollectionCertStoreParameters par =
            new CollectionCertStoreParameters(cert);
    final CertStore store = CertStore.getInstance("Collection", par);
    cpp.addCertStore(store);
    final X509CertSelector selector = new X509CertSelector();
    selector.setCertificate((X509Certificate) crt);
    cpp.setTargetCertConstraints(selector);

    // (CertPath)
    //1)    (  )
    //final CertificateFactory cf = CertificateFactory.getInstance("X509");
    //final CertPath cp = cf.generateCertPath(cert);

    //2)  
    //)   crl
    //cpp.setRevocationEnabled(true);
    //    CRL Distribution Points
    // System.setProperty("com.sun.security.enableCRLDP", "true");
    // System.setProperty("com.ibm.security.enableCRLDP", "true");

    //)   crl
    cpp.setRevocationEnabled(false);
    final PKIXCertPathBuilderResult res =
            (PKIXCertPathBuilderResult) CertPathBuilder.
                    getInstance("PKIX").build(cpp);
    final CertPath cp = res.getCertPath();

    //
    final CertPathValidator cpv = CertPathValidator.getInstance("PKIX");
    cpp.setRevocationEnabled(true);
    cpv.validate(cp, cpp);
}
}
