/**
 * $RCSfile: SignAndVerify.java,v $
 * version $Revision: 1.15 $
 * created 14.04.2005 18:01:52 by elvira
 * last modified $Date: 2011/02/10 12:17:52 $ by $Author: iva $
 * (C)  - 2004-2009.
 *
 *  ,    , 
 *   .     
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package userSamples;

import ru.CryptoPro.JCP.params.DigestParamsSpec;
import ru.CryptoPro.JCP.params.OID;
import ru.CryptoPro.JCP.params.ParamsInterface;
import ru.CryptoPro.JCP.tools.Platform;

import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;

/**
 *           
 *    34.10-2001.
 */
public class SignAndVerify {
/**
 * 
 */
private static final String SAMPLE_TEXT = "generating and verifing signature";

/**
 * @param args null
 * @throws Exception /
 */
public static void main(String[] args) throws Exception {
    //*   
    final KeyPair keyPair = KeyPairGen.genKey(Constants.SIGN_KEY_PAIR_ALG);

    //*   "GOST3411withGOST3410EL"
    final byte[] signEL = sign(Constants.SIGN_EL_ALG, keyPair.getPrivate(),
            SAMPLE_TEXT.getBytes());
    System.out.println("Value of signature (signEL) is:");
    System.out.println(Constants.toHexString(signEL));
    //  
    final boolean signELver = verify(Constants.SIGN_EL_ALG, keyPair.getPublic(),
            SAMPLE_TEXT.getBytes(), signEL);
    System.out.println("Signature verifies (signEL) is: " + signELver);

    //*   "CryptoProSignature" (   CSP)
    final byte[] signCP = sign(Constants.SIGN_CP_ALG, keyPair.getPrivate(),
            SAMPLE_TEXT.getBytes());
    System.out.println("Value of signature (signCP) is:");
    System.out.println(Constants.toHexString(signCP));
    //  
    final boolean signCPver = verify(Constants.SIGN_CP_ALG, keyPair.getPublic(),
            SAMPLE_TEXT.getBytes(), signCP);
    System.out.println("Signature verifies (signCP) is: " + signCPver);

    //*      

    // !      
    //      :
    // DigestParamsSpec.OID_HashVerbaO ("1.2.643.2.2.30.1")

    final OID digestOid = DigestParamsSpec.OID_HashVar_1; //"1.2.643.2.2.30.2";
    final ParamsInterface digestParams =
            DigestParamsSpec.getInstance(digestOid);

    //
    final byte[] sign = sign(Constants.SIGN_EL_ALG, keyPair.getPrivate(),
            SAMPLE_TEXT.getBytes(), digestParams);
    System.out.println("Value of signature (sign) is:");
    System.out.println(Constants.toHexString(sign));
    //      
    final boolean ver = verify(Constants.SIGN_EL_ALG, keyPair.getPublic(),
            SAMPLE_TEXT.getBytes(), sign, digestParams);
    System.out.println("Signature verifies (sign) is: " + ver);

    System.out.println("OK");
}

/**
 *  
 *
 * @param alghorithmName  
 * @param privateKey  
 * @param data  
 * @return 
 * @throws Exception /
 */
public static byte[] sign(String alghorithmName, PrivateKey privateKey,
                          byte[] data) throws Exception {
    final Signature sig = Signature.getInstance(alghorithmName);
    sig.initSign(privateKey);
    sig.update(data);
    return sig.sign();
}

/**
 *      
 *
 * @param alghorithmName  
 * @param privateKey  
 * @param data  
 * @param digestParams  
 * @return 
 * @throws Exception /
 */
public static byte[] sign(String alghorithmName, PrivateKey privateKey,
                          byte[] data, ParamsInterface digestParams)
        throws Exception {
    final Signature sig = Signature.getInstance(alghorithmName);
    sig.initSign(privateKey);
    /**
     * Java  IBM    .
     */
    if (!Platform.isIBM()) {
        sig.setParameter(digestParams);
    }
    sig.update(data);
    return sig.sign();
}

/**
 *     
 *
 * @param alghorithmName  
 * @param publicKey  
 * @param data  
 * @param signature 
 * @return true - , false -  
 * @throws Exception /
 */
public static boolean verify(String alghorithmName, PublicKey publicKey,
                             byte[] data, byte[] signature) throws Exception {
    final Signature sig = Signature.getInstance(alghorithmName);
    sig.initVerify(publicKey);
    sig.update(data);
    return sig.verify(signature);
}

/**
 *         
 *
 * @param alghorithmName  
 * @param publicKey  
 * @param data  
 * @param signature 
 * @param digestParams  
 * @return true - , false -  
 * @throws Exception /
 */
public static boolean verify(String alghorithmName, PublicKey publicKey,
                             byte[] data, byte[] signature,
                             ParamsInterface digestParams) throws Exception {
    final Signature sig = Signature.getInstance(alghorithmName);
    sig.initVerify(publicKey);
    sig.setParameter(digestParams);
    sig.update(data);
    return sig.verify(signature);
}
}
