/**
 * $RCSfile: SpeedTest.java,v $
 * version $Revision: 1.1 $
 * created 13.10.2008 14:47:53 by Iva
 * last modified $Date: 2008/10/13 13:35:38 $ by $Author: iva $
 * (C)  - 2004-2008.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package userSamples;

import ru.CryptoPro.Crypto.CryptoProvider;
import ru.CryptoPro.JCP.JCP;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.ProviderException;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.util.Arrays;

/**
 *   JCP.
 *
 * @author Copyright 2004-2008 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class SpeedTest {
/**
 * size of byte array.
 */
public static final int BLOCK_SIZE = 16384;
/**
 * repeate amount.
 */
public static final int LOOPS_AMOUNT = 1000;

/**
 * Static class, ctor forbidden.
 */
private SpeedTest() {
}

/**
 * main.
 *
 * @param args not used.
 * @throws Exception any error
 */
public static void main(String[] args) throws Exception {

    byte[] testText = new byte[8];
    byte[] longText = new byte[BLOCK_SIZE];
    long startTime;
    long endTime;
    // create objects
    MessageDigest digest = MessageDigest.getInstance(JCP.GOST_DIGEST_NAME);
    SecureRandom random = SecureRandom.getInstance(JCP.CP_RANDOM);
    IvParameterSpec params = new IvParameterSpec(testText);
    Cipher cipher = Cipher.getInstance("GOST28147/CFB/NoPadding");
    // generate keys
    KeyGenerator kg = KeyGenerator.getInstance(CryptoProvider.GOST_CIPHER_NAME);
    SecretKey key = kg.generateKey();
    KeyPairGenerator keyGen =
            KeyPairGenerator.getInstance(CryptoProvider.GOST_EPH_DH_NAME);
    java.security.KeyPair pair = keyGen.generateKeyPair();
    PrivateKey privateKey = pair.getPrivate();
    PublicKey publicKey = pair.getPublic();
    // test create sign
    Signature signature = Signature.getInstance(JCP.GOST_EL_SIGN_NAME);
    signature.initSign(privateKey);
    signature.update(testText);
    byte[] signBytes = signature.sign();
    // test verify sign
    signature.initVerify(publicKey);
    signature.update(testText);
    if (!signature.verify(signBytes))
        throw new ProviderException();
    // test encrypt
    cipher.init(Cipher.ENCRYPT_MODE, key, params);
    byte[] encryptResult = cipher.doFinal(testText);
    cipher.init(Cipher.DECRYPT_MODE, key, params);
    byte[] decryptResult = cipher.doFinal(encryptResult);
    if (!Arrays.equals(decryptResult, testText))
        throw new ProviderException();

    // test random speed
    random.nextBytes(longText);
    startTime = System.currentTimeMillis();
    for (int i = 0; i < LOOPS_AMOUNT; i++) {
        random.nextBytes(longText);
    }
    endTime = System.currentTimeMillis();
    long randomSpeed =
            ((long) BLOCK_SIZE * LOOPS_AMOUNT) / (endTime - startTime);

    // test digest speed.
    digest.update(longText);
    startTime = System.currentTimeMillis();
    for (int i = 0; i < LOOPS_AMOUNT; i++) {
        digest.update(longText);
    }
    endTime = System.currentTimeMillis();
    long digestSpeed =
            ((long) BLOCK_SIZE * LOOPS_AMOUNT) / (endTime - startTime);

    // test encrypt speed
    cipher.init(Cipher.ENCRYPT_MODE, key, params);
    cipher.update(longText);
    startTime = System.currentTimeMillis();
    for (int i = 0; i < LOOPS_AMOUNT; i++) {
        cipher.update(longText);
    }
    endTime = System.currentTimeMillis();
    long cryptSpeed =
            ((long) BLOCK_SIZE * LOOPS_AMOUNT) / (endTime - startTime);

    // test sign speed
    signature.initSign(privateKey);
    signature.update(testText);
    signature.sign();
    startTime = System.currentTimeMillis();
    for (int i = 0; i < LOOPS_AMOUNT; i++) {
        signature.initSign(privateKey);
        signature.update(testText);
        signature.sign();
    }
    endTime = System.currentTimeMillis();
    long signTime = endTime - startTime;

    // test verify speed
    signature.initVerify(publicKey);
    signature.update(testText);
    signature.verify(signBytes);
    startTime = System.currentTimeMillis();
    for (int i = 0; i < LOOPS_AMOUNT; i++) {
        signature.initVerify(publicKey);
        signature.update(testText);
        signature.verify(signBytes);
    }
    endTime = System.currentTimeMillis();
    long verifyTime = endTime - startTime;

    System.out.println("Random Speed:" + randomSpeed + " KB per sec.");
    System.out.println("Digest Speed:" + digestSpeed + " KB per sec.");
    System.out.println("Crypt speed:" + cryptSpeed + " KB per sec.");
    System.out.println("Signature generation time: " +
            signTime / LOOPS_AMOUNT + "." + signTime % LOOPS_AMOUNT);
    System.out.println("Signature verify time: " +
            verifyTime / LOOPS_AMOUNT + "." + verifyTime % LOOPS_AMOUNT);
}
}
