/**
 * $RCSfile: XWSXmlDSig.java,v $
 * version $Revision: 1.6 $
 * created 28.02.2005 21:10:08 by avsh
 * last modified $Date: 2008/10/29 11:36:08 $ by $Author: kunina $
 *  (C)  - 2004-2005.
 *
 *  ,    , 
 *   .     
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package userSamples;

import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.ObjectContainer;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.utils.Constants;
import org.apache.xml.security.utils.XMLUtils;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;

/**
 *  XML .
 */

public class XWSXmlDSig {
private String signMethod;      // 
private String digestMethod;    //  
private String transform;       //    
private X509Certificate cert;   //  ,   (   
//     )
private PrivateKey privateKey;  //      

//      

public String getSignMethod() {
    return signMethod;
}

public String getDigestMethod() {
    return signMethod;
}

public void setSignMethod(String SM) {
    signMethod = SM;
}

public void setdigestMethod(String DM) {
    digestMethod = DM;
}

public void SetCertificate(X509Certificate Cer) {
    cert = Cer;
    //            
    //    RSA       SHA1
    if (algEquals(cert.getPublicKey().getAlgorithm(),
            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1)) {
        signMethod = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;
        digestMethod = Constants.ALGO_ID_DIGEST_SHA1;
    }
    if (algEquals(cert.getPublicKey().getAlgorithm(),
            XMLSignature.ALGO_ID_SIGNATURE_DSA)) {
        signMethod = XMLSignature.ALGO_ID_SIGNATURE_DSA;
        digestMethod = Constants.ALGO_ID_DIGEST_SHA1;
    }
}

public X509Certificate getCertificate() {
    return cert;
}

public PrivateKey getPrivateKey() {
    return privateKey;
}

public void setPrivateKey(PrivateKey pKey) {
    privateKey = pKey;
}

public XWSXmlDSig() {// -     RSA   SHA1  
    //     ENVELOPED
    signMethod = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;
    digestMethod = Constants.ALGO_ID_DIGEST_SHA1;
    transform = Transforms.TRANSFORM_C14N_WITH_COMMENTS;
}

static {
    ru.CryptoPro.JCPxml.xmldsig.JCPXMLDSigInit.init();
}

//   

public Document SignObject() throws Exception {

    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    Document doc = dbf.newDocumentBuilder().newDocument();


    XMLSignature sig = new XMLSignature(doc, "", signMethod);
    doc.appendChild(sig.getElement()); //  


    ObjectContainer obj = new ObjectContainer(doc);
    Element anElement = doc.createElement(
            "InsideObject");   //     

    anElement.appendChild(doc.createTextNode("A text in a box"));
    obj.appendChild(anElement);

    String Id = "TheFirstObject";

    obj.setId(Id);
    sig.appendObject(obj);


    Transforms transforms = new Transforms(doc);
    transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
    transforms.addTransform(transform);

    sig.addDocument("", transforms,
            digestMethod);  //  ref ( =>   -)


    sig.addKeyInfo(cert);

    sig.sign(privateKey);
    return doc;
}

//    

public static boolean Validate(Document doc)
        throws TransformerException, XMLSecurityException {
    Element nscontext = XMLUtils
            .createDSctx(doc, "ds", Constants.SignatureSpecNS);

    Element sigElement = (Element) XPathAPI.selectSingleNode(doc,
            "//ds:Signature[1]", nscontext);
    XMLSignature signature = new XMLSignature(sigElement, "");

    KeyInfo ki = signature.getKeyInfo();

    if (ki != null) {
        if (ki.containsX509Data()) {
            System.out
                    .println("Could find a X509Data element in the KeyInfo");
        }

        X509Certificate cert = signature.getKeyInfo().getX509Certificate();

        if (cert != null) {

            System.out.println("The XML signature  is "
                    + (signature.checkSignatureValue(cert)
                    ? "valid (good)"
                    : "invalid !!!!! (bad)"));
        } else {
            System.out.println("Did not find a Certificate");

            PublicKey pk = signature.getKeyInfo().getPublicKey();

            if (pk != null) {

                System.out.println(
                        "I try to verify the signature using the public key: "
                                + pk);

                System.out.println("The XML signature is "
                        + (signature.checkSignatureValue(pk)
                        ? "valid (good)"
                        : "invalid !!!!! (bad)"));
            } else {
                System.out.println(
                        "Did not find a public key, so I can't check the signature");
                return false;
            }
        }
    } else {
        System.out.println("Did not find a KeyInfo");
        return false;
    }

    return true;
}


static boolean algEquals(String algURI, String algName) {
    if (algName.equalsIgnoreCase("DSA") &&
            algURI.equalsIgnoreCase(XMLSignature.ALGO_ID_SIGNATURE_DSA)) {
        return true;
    } else if (algName.equalsIgnoreCase("RSA") &&
            algURI.equalsIgnoreCase(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1)) {
        return true;
    } else if (algName.equalsIgnoreCase("1.2.643.2.2.20") &&
            algURI.equalsIgnoreCase(
                    "http://www.w3.org/2001/04/xmldsig-more#gostr34101994-gostr3411")) {
        return true;
    } else {
        return false;
    }
}
}

