/**
 * $RCSfile: CryptXML.java,v $
 * version $Revision: 1.1 $
 * created 13.07.2009 16:38:35 by Iva
 * last modified $Date: 2009/07/13 12:59:26 $ by $Author: iva $
 * (C)  - 2004-2009.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package xmlSign;

import org.apache.xml.security.encryption.EncryptedData;
import org.apache.xml.security.encryption.EncryptedKey;
import org.apache.xml.security.encryption.XMLCipher;
import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.keys.content.X509Data;
import org.apache.xml.security.utils.Constants;
import org.apache.xml.security.utils.EncryptionConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.CryptoPro.JCP.Key.KeyTwix;
import ru.CryptoPro.JCPRequest.CertGen;
import ru.CryptoPro.JCPxml.Consts;
import ru.CryptoPro.JCPxml.XmlInit;

import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.OutputStream;
import java.security.Key;
import java.security.cert.X509Certificate;

/**
 *   XML.
 *
 * @author Copyright 2004-2009 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class CryptXML {
public static void main(String[] args) throws Exception {
    /**
     *   
     */
    XmlInit.init();

    /**
     *   XML   .
     */
    Document doc = createSampleDocument();
    writeDoc(doc, System.out);
    System.out.println("");

    /*
     *      .
     */
    KeyTwix twix = CertGen.generateTwix("GOST3410DHEPH",
            "GOST3411withGOST3410DHEL",
            "CN=cert");
    //KeyTwix twix = new KeyTwix("XmlSignEncrypt");

    /**
     *  " ".
     */
    encrypt(doc, twix.getCert());
    writeDoc(doc, System.out);
    System.out.println("");

    /**
     *    .
     */
    decrypt(doc, twix.getPrivate());
    writeDoc(doc, System.out);
    System.out.println("");

}

/**
 *  
 *
 * @param doc ,   
 * @param cert 
 * @return  
 * @throws Exception  
 */
public static Document encrypt(Document doc, X509Certificate cert)
        throws Exception {
    /**
     *    .
     */
    SecretKey sessionKey = KeyGenerator.getInstance("GOST28147").generateKey();
    /**
     *   .
     */
    EncryptedKey encryptedKey = wrapKey(doc, sessionKey, cert);
    /**
     *  
     */
    return encrypt(doc, sessionKey, encryptedKey);
}

/**
 *   doc  sessionKey.
 *
 * @param doc ,   
 * @param sessionKey   
 * @param encryptedKey  sessionKey    
 * @return  
 * @throws Exception  
 */
public static Document encrypt(Document doc, SecretKey sessionKey,
                               EncryptedKey encryptedKey) throws Exception {
    Element element = doc.getDocumentElement();
    /**
     *     .  URI_GOST_CIPHER
     *    ru.CryptoPro.JCPxml.Consts
     * public static final String URI_GOST_CIPHER =
     * "urn:ietf:params:xml:ns:cpxmlsec:algorithms:gost28147";
     */
    XMLCipher xmlCipher = XMLCipher.getInstance(Consts.URI_GOST_CIPHER);
    xmlCipher.init(XMLCipher.ENCRYPT_MODE, sessionKey);
    /**
     *   .
     */
    EncryptedData encryptedData = xmlCipher.getEncryptedData();
    KeyInfo keyInfo = new KeyInfo(doc);
    keyInfo.add(encryptedKey);
    encryptedData.setKeyInfo(keyInfo);
    /**
     *  
     */
    xmlCipher.doFinal(doc, element, true);
    return doc;
}

/**
 *    <code>sessionKey</code>  
 * <code>EncryptedKey</code>  .
 *
 * @param doc xml 
 * @param sessionKey   .
 * @param cert 
 * @return  
 * @throws Exception  
 */
public static EncryptedKey wrapKey(Document doc, SecretKey sessionKey,
                                   X509Certificate cert)
        throws Exception {
    /**
     *     .  URI_GOST_TRANSPORT
     *    ru.CryptoPro.JCPxml.Consts
     * public static final String URI_GOST_TRANSPORT =
     * "urn:ietf:params:xml:ns:cpxmlsec:algorithms:transport-gost2001";
     */
    XMLCipher keyCipher = XMLCipher.getInstance(Consts.URI_GOST_TRANSPORT);
    keyCipher.init(XMLCipher.WRAP_MODE, cert.getPublicKey());
    /**
     *  KeyInfo  
     */
    KeyInfo certKeyInfo = new KeyInfo(doc);
    X509Data x509data = new X509Data(doc);
    x509data.addCertificate(cert);
    certKeyInfo.add(x509data);
    /**
     *  
     */
    EncryptedKey encryptedKey = keyCipher.encryptKey(doc, sessionKey);
    encryptedKey.setKeyInfo(certKeyInfo);
    return encryptedKey;
}

/**
 *  .
 *
 * @param doc  
 * @param key   
 * @return  
 * @throws Exception error
 */
public static Document decrypt(Document doc, Key key) throws Exception {
    /**
     * create cipher in decrypt mode.
     */
    XMLCipher xmlCipher = XMLCipher.getInstance();
    xmlCipher.init(XMLCipher.DECRYPT_MODE, null);
    /*
     * extract element to decrypt.
     */
    Element encryptedDataElement = (Element) doc.getElementsByTagNameNS(
            EncryptionConstants.EncryptionSpecNS,
            EncryptionConstants._TAG_ENCRYPTEDDATA).item(0);
    /*
     * The key to be used for decrypting xml data would be obtained
     * from the keyinfo of the EncrypteData using the kek.
     */
    if (key != null)
        xmlCipher.setKEK(key);
    /*
     * The following doFinal call replaces the encrypted data with
     * decrypted contents in the document.
     */
    xmlCipher.doFinal(doc, encryptedDataElement);
    return doc;
}

/**
 * create DocumentBuilderFactory with properties.
 *
 * @return DocumentBuilderFactory
 */
public static DocumentBuilderFactory createDocFactory() {
    //    XML-
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    //  ,   
    //      XML-
    dbf.setIgnoringElementContentWhitespace(true);
    //  ,    CDATA
    //      XML-
    dbf.setCoalescing(true);
    //  ,    
    //   XML-
    dbf.setNamespaceAware(true);

    return dbf;
}

/**
 * create new simple document to test. <p/> It will look something like:
 * <p/>
 * <apache:RootElement xmlns:apache="http://www.apache.org/ns/#app1">
 * <apache:foo>Some simple text</apache:foo> </apache:RootElement>
 *
 * @return test document
 * @throws javax.xml.parsers.ParserConfigurationException if a DocumentBuilder
 * cannot be created which satisfies the configuration requested.
 */
public static Document createSampleDocument()
        throws ParserConfigurationException {

    DocumentBuilderFactory dbf = createDocFactory();
    Document document = dbf.newDocumentBuilder().newDocument();

    Element root = document.createElementNS("http://www.apache.org/ns/#app1",
            "apache:RootElement");
    root.setAttributeNS(Constants.NamespaceSpecNS, "xmlns:apache",
            "http://www.apache.org/ns/#app1");
    document.appendChild(root);

    root.appendChild(document.createTextNode("\n"));

    Element childElement = document.createElementNS(
            "http://www.apache.org/ns/#app1", "apache:foo");
    childElement.appendChild(document.createTextNode("Some simple text"));
    root.appendChild(childElement);

    root.appendChild(document.createTextNode("\n"));

    return document;
}

/**
 * save doc.
 *
 * @param doc document to save.
 * @param out output stream
 * @throws javax.xml.transform.TransformerException If an unrecoverable error
 * occurs during the course of the transformation.
 */
public static void writeDoc(Document doc, OutputStream out)
        throws TransformerException {
    //     XML-  
    Transformer transformer = TransformerFactory.newInstance().newTransformer();
    transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
    //   XML-  
    transformer.transform(new DOMSource(doc), new StreamResult(out));
}

}
