/**
 * $RCSfile: XMLSignDoc.java,v $
 * version $Revision: 1.4 $
 * created 13.08.2007 14:09:47 by kunina
 * last modified $Date: 2008/10/29 11:34:12 $ by $Author: kunina $
 * (C)  - 2004-2007.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package xmlSign;

import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.utils.Constants;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.CryptoPro.JCPRequest.GostCertificateRequest;
import ru.CryptoPro.JCPxml.xmldsig.JCPXMLDSigInit;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.logging.Logger;

/**
 *      XML-    
 * 34.10-2001.
 *
 * @author Copyright 2004-2007 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class XMLSignDoc {
 /**/
private XMLSignDoc() {
}

public static void main(String[] args) {
    try {
        // xml 
        final String testDoc = "tests.xml";

        // xml 
        final String signDoc = "XmlDSigDocument.xml";

        //    34.10-2001 (   )
        final String KeyPairAlgorithm = "GOST3410";

        //   (  )    
        final String certName = "CN=newCert, O=CryptoPro, C=RU";

        //   (  34.10-2001)
        final String signMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr34102001-gostr3411";

        //  ,    (  34.11-94)
        final String digestMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr3411";

        /*          34.10-2001*/
        JCPXMLDSigInit.init();

        Logger.getLogger("LOG").info("sign doc begin");
        signDoc(KeyPairAlgorithm, certName, signMethod, digestMethod, testDoc,
                signDoc);
        Logger.getLogger("LOG").info("sign doc end\nsign doc verify");
        signDocVer(signDoc);

    } catch (Exception e) {
        e.printStackTrace();
    }
}

/**
 *    XML-     34.10-2001.
 *
 * @param keyPairAlgorithm    34.10-2001 (  
 * )
 * @param certName   (  )   
 * 
 * @param signMethod   (  34.10-2001)
 * @param digestMethod  ,    ( 
 * 34.11-94)
 * @param testDoc  
 * @param signDoc  
 * @throws Exception e
 */
public static void signDoc(String keyPairAlgorithm, String certName,
                           String signMethod, String digestMethod,
                           String testDoc, String signDoc)
        throws Exception {
    /*           XML-*/

    //     
    final KeyPairGenerator keyGen =
            KeyPairGenerator.getInstance(keyPairAlgorithm);

    //   
    final KeyPair keypair = keyGen.generateKeyPair();

    //   
    final PublicKey publicKey = keypair.getPublic();

    //   
    final PrivateKey privateKey = keypair.getPrivate();

    /*        */

    //    
    final GostCertificateRequest request = new GostCertificateRequest();

    //   ,   DER-
    final byte[] encodedCert = request.getEncodedSelfCert(keypair, certName);

    //   X509-
    final CertificateFactory cf = CertificateFactory.getInstance("X509");

    //  X509-    
    final X509Certificate cert = (X509Certificate) cf
            .generateCertificate(new ByteArrayInputStream(encodedCert));

    /*   XML-   */

    //    XML-
    final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,         XML-
    dbf.setIgnoringElementContentWhitespace(true);

    //  ,    CDATA      XML-
    dbf.setCoalescing(true);

    //  ,       XML-
    dbf.setNamespaceAware(true);

    //         
    final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
    final Document doc = documentBuilder.parse(testDoc);

    /*    <ds:Signature>   XML- */

    //           34.10-2001
    final XMLSignature sig = new XMLSignature(doc, "", signMethod);

    //    XML-
    final Element anElement = doc.getDocumentElement();

    //     XML-  
    anElement.appendChild(sig.getElement());

    /*     XML-        */

    //    <ds:Transforms>  XML-
    final Transforms transforms = new Transforms(doc);

    //        
    transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
    transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);

    //      ( <ds:Reference>),    
    // XML- (       <ds:Transforms> 
    //    )
    sig.addDocument("", transforms, digestMethod);

    /*     XML-    ,     */

    //      <ds:KeyInfo>      
    // 
    sig.addKeyInfo(cert);

    //   XML-
    sig.sign(privateKey);

    /*   XML-   */

    //  ,      XML-
    final FileOutputStream os = new FileOutputStream(signDoc);

    //     XML-  
    final TransformerFactory tf = TransformerFactory.newInstance();

    //     XML-  
    final Transformer trans = tf.newTransformer();

    //   XML-  
    trans.transform(new DOMSource(doc), new StreamResult(os));
    os.close();
}

/**
 *    XML-     34.10-2001.
 *
 * @param signDoc  
 * @throws Exception /
 */
public static void signDocVer(String signDoc) throws Exception {

    //    XML-
    final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,         XML-
    dbf.setIgnoringElementContentWhitespace(true);

    //  ,    CDATA      XML-
    dbf.setCoalescing(true);

    //  ,       XML-
    dbf.setNamespaceAware(true);

    //         
    final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();

    /*   XML-   */

    final Document doc = documentBuilder.parse(signDoc);

    /*    <ds:Signature>  XML- */

    //        Signature
    final Element nscontext = doc.createElementNS(null, "namespaceContext");
    nscontext.setAttributeNS("http://www.w3.org/2000/xmlns/",
            "xmlns:" + "ds".trim(), Constants.SignatureSpecNS);

    //         <ds:Signature>
    final Element sigElement = (Element) XPathAPI
            .selectSingleNode(doc, "//ds:Signature[1]", nscontext);

    /*   XML-      ,  
    XML- */

    //    
    final XMLSignature signature = new XMLSignature(sigElement, "");

    //   <ds:KeyInfo>    
    final KeyInfo ki = signature.getKeyInfo();

    //        
    final X509Certificate certKey = ki.getX509Certificate();

    //   ,   
    //    
    if (certKey != null) {
        Logger.getLogger("LOG").info("The XML signature  is " +
                (signature.checkSignatureValue(certKey)
                        ? "valid (good)" : "invalid (bad)"));

    }
    //        
    else {
        //         
        final PublicKey pk = ki.getPublicKey();

        //    ,      
        if (pk != null) {
            Logger.getLogger("LOG").info(
                    "The XML signature is " + (signature.checkSignatureValue(pk)
                            ? "valid (good)" : "invalid (bad)"));
        }
        //        
        else
            throw new Exception(
                    "There are no information about public key. Verification couldn't be implemented");

    }
}
}
