/**
 * $RCSfile: XMLSignObj.java,v $
 * version $Revision: 1.4 $
 * created 13.08.2007 14:07:04 by kunina
 * last modified $Date: 2008/10/29 11:34:12 $ by $Author: kunina $
 * (C)  - 2004-2007.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package xmlSign;

import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.ObjectContainer;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.utils.Constants;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.CryptoPro.JCPRequest.GostCertificateRequest;
import ru.CryptoPro.JCPxml.xmldsig.JCPXMLDSigInit;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.logging.Logger;


/**
 *      XML-    
 * 34.10-2001.
 *
 * @author Copyright 2004-2007 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class XMLSignObj {
 /**/
private XMLSignObj() {
}

public static void main(String[] args) {
    try {
        // xml 
        final String signObj = "XmlDSigObject.xml";

        //    34.10-2001 (   )
        final String KeyPairAlgorithm = "GOST3410";

        //   (  )    
        final String certName = "CN=newCert, O=CryptoPro, C=RU";

        //   
        final String Id = "TheFirstObject";

        //   (  34.10-2001)
        final String signMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr34102001-gostr3411";

        //  ,    (  34.11-94)
        final String digestMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr3411";

        /*          34.10-2001*/
        JCPXMLDSigInit.init();

        Logger.getLogger("LOG").info("sign obj begin");
        signObj(KeyPairAlgorithm, certName, Id, signMethod, digestMethod,
                signObj);
        Logger.getLogger("LOG").info("sign obj end\nsign obj verify");
        signObjVer(signObj);

    } catch (Exception e) {
        e.printStackTrace();
    }


}

/**
 *    XML-     34.10-2001.
 *
 * @param keyPairAlgorithm    34.10-2001 (  
 * )
 * @param certName   (  )   
 * 
 * @param id   
 * @param signMethod   (  34.10-2001)
 * @param digestMethod  ,    ( 
 * 34.11-94)
 * @param signObj  
 * @throws Exception e
 */
public static void signObj(String keyPairAlgorithm, String certName,
                           String id, String signMethod,
                           String digestMethod, String signObj)
        throws Exception {
/*           XML-*/

    //     
    final KeyPairGenerator keyGen =
            KeyPairGenerator.getInstance(keyPairAlgorithm);

    //   
    final KeyPair keypair = keyGen.generateKeyPair();

    //   
    final PublicKey publicKey = keypair.getPublic();

    //   
    final PrivateKey privateKey = keypair.getPrivate();

/*        */

    //    
    final GostCertificateRequest request = new GostCertificateRequest();

    //   ,   DER-
    final byte[] encodedCert = request.getEncodedSelfCert(keypair, certName);

    //   X509-
    final CertificateFactory cf = CertificateFactory.getInstance("X509");

    //  X509-    
    final X509Certificate cert = (X509Certificate) cf
            .generateCertificate(new ByteArrayInputStream(encodedCert));

/*   () XML- */

    //    XML-
    final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,       XML-
    dbf.setNamespaceAware(true);

    //   () XML-
    final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
    final Document doc = documentBuilder.newDocument();

/*    <ds:Signature>   XML- */

    //           34.10-2001
    final XMLSignature sig = new XMLSignature(doc, "", signMethod);

    //      XML-
    doc.appendChild(sig.getElement());

/*       */

    //    <ds:Object>   XML- (  ,  
    //     )
    final ObjectContainer obj = new ObjectContainer(doc);

    //      "InsideObject"
    final Element anElement = doc.createElement("InsideObject");

    //       "InsideObject"
    anElement.appendChild(doc.createTextNode("A text in a box"));

    //     
    obj.appendChild(anElement);

    //    
    obj.setId(id);

    //       
    sig.appendObject(obj);

/*     XML-        */

    //    <ds:Transforms>  XML-
    final Transforms transforms = new Transforms(doc);

    //        
    transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
    transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);

    //      ( <ds:Reference>),    
    // XML- (       <ds:Transforms> 
    //    )
    sig.addDocument("", transforms, digestMethod);

/*    XML-    ,     */

    //      <ds:KeyInfo>      
    // 
    sig.addKeyInfo(cert);

    //    XML-
    sig.sign(privateKey);

/*   XML- (   XML-)   */

    //  ,      XML-
    final FileOutputStream os = new FileOutputStream(signObj);

    //     XML-  
    final TransformerFactory tf = TransformerFactory.newInstance();

    //     XML-  
    final Transformer trans = tf.newTransformer();

    //   XML-  
    trans.transform(new DOMSource(doc), new StreamResult(os));
    os.close();
}

/**
 *    XML-     34.10-2001.
 *
 * @param signObj  
 * @throws Exception /
 */
public static void signObjVer(String signObj) throws Exception {
    //    XML-
    final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,       XML-
    dbf.setNamespaceAware(true);

/*   XML- (    XML-)   */

    final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
    final Document doc = documentBuilder.parse(signObj);

/*    <ds:Signature>  XML- */

//        Signature
    final Element nscontext = doc.createElementNS(null, "namespaceContext");
    nscontext.setAttributeNS("http://www.w3.org/2000/xmlns/",
            "xmlns:" + "ds".trim(), Constants.SignatureSpecNS);

//         <ds:Signature>
    final Element sigElement = (Element) XPathAPI
            .selectSingleNode(doc, "//ds:Signature[1]", nscontext);

/*    XML-      ,  
XML- */

//    
    final XMLSignature signature = new XMLSignature(sigElement, "");

//   <ds:KeyInfo>    
    final KeyInfo ki = signature.getKeyInfo();

//        
    final X509Certificate certKey = ki.getX509Certificate();

//   ,   
//    
    if (certKey != null) {
        Logger.getLogger("LOG").info("The XML signature  is " +
                (signature.checkSignatureValue(certKey)
                        ? "valid (good)" : "invalid (bad)"));
    }
//        
    else {
//         
        final PublicKey pk = ki.getPublicKey();

//    ,      
        if (pk != null) {
            Logger.getLogger("LOG").info(
                    "The XML signature is " + (signature.checkSignatureValue(pk)
                            ? "valid (good)" : "invalid (bad)"));
        }
        //        
        else throw new Exception(
                "There are no information about public key. Verification couldn't be implemented");
    }
}
}
