/**
 * $RCSfile: XMLTwoSignDoc.java,v $
 * version $Revision: 1.5 $
 * created 13.08.2007 14:27:59 by kunina
 * last modified $Date: 2011/03/15 13:46:42 $ by $Author: iva $
 * (C)  - 2004-2007.
 *
 *  ,    , 
 *   .      
 *         .
 *
 *       
 *   .  -   
 *     .
 */
package xmlSign;

import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.transforms.params.XPath2FilterContainer;
import org.apache.xml.security.utils.Constants;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.CryptoPro.JCPRequest.GostCertificateRequest;
import ru.CryptoPro.JCPxml.xmldsig.JCPXMLDSigInit;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.logging.Logger;


/**
 *        XML- 
 *    34.10-2001.
 *
 * @author Copyright 2004-2007 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class XMLTwoSignDoc {
 /**/
private XMLTwoSignDoc() {
}

public static void main(String[] args) {
    try {
        // xml 
        String testDoc = "tests.xml";

        // xml 
        String signTwo = "XmlTwoDSig.xml";

        //    34.10-2001 (   )
        String KeyPairAlgorithm = "GOST3410";

        //   (  )     
        String certName1 = "CN=newCert1, O=CryptoPro, C=RU";

        //   (  )     
        String certName2 = "CN=newCert2, O=CryptoPro, C=RU";

        //    
        String Id1 = "FirstSignature";

        //    
        String Id2 = "SecondSignature";

        //   (  34.10-2001)
        String signMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr34102001-gostr3411";

        //  ,    (  34.11-94)
        String digestMethod =
                "http://www.w3.org/2001/04/xmldsig-more#gostr3411";

        /*          34.10-2001*/
        JCPXMLDSigInit.init();

        Logger.getLogger("LOG").info("two sign doc begin");
        twoSignDoc(KeyPairAlgorithm, certName1, certName2, Id1, Id2, signMethod,
                digestMethod, testDoc, signTwo);
        Logger.getLogger("LOG").info("two sign doc end\ntwo sign doc verify");
        twoSignDocVer(signTwo);

    } catch (Exception e) {
        e.printStackTrace();
    }

}

/**
 * @param keyPairAlgorithm    34.10-2001 ( 
 *  )
 * @param certName1   (  )   
 *  
 * @param certName2   (  )   
 *  
 * @param Id1    
 * @param Id2    
 * @param signMethod   (  34.10-2001)
 * @param digestMethod  ,    ( 
 * 34.11-94)
 * @param testDoc  
 * @param signTwo  
 * @throws Exception e
 */
public static void twoSignDoc(String keyPairAlgorithm, String certName1,
                              String certName2, String Id1, String Id2,
                              String signMethod, String digestMethod,
                              String testDoc, String signTwo)
        throws Exception {
    /*              XML-*/

    //     
    KeyPairGenerator keyGen = KeyPairGenerator.getInstance(keyPairAlgorithm);

    //    
    KeyPair keypair1 = keyGen.generateKeyPair();

    //      
    PublicKey publicKey1 = keypair1.getPublic();

    //      
    PrivateKey privateKey1 = keypair1.getPrivate();

    //    
    KeyPair keypair2 = keyGen.generateKeyPair();

    //      
    PublicKey publicKey2 = keypair2.getPublic();

    //      
    PrivateKey privateKey2 = keypair2.getPrivate();

    /*          */

    //    
    GostCertificateRequest request1 = new GostCertificateRequest();
    GostCertificateRequest request2 = new GostCertificateRequest();

    //    ,   DER-
    byte[] encodedCert1 = request1.getEncodedSelfCert(keypair1, certName1);

    //   X509-
    CertificateFactory cf = CertificateFactory.getInstance("X509");

    //  X509-     
    X509Certificate cert1 = (X509Certificate) cf
            .generateCertificate(new ByteArrayInputStream(encodedCert1));

    //    ,   DER-
    byte[] encodedCert2 = request2.getEncodedSelfCert(keypair2, certName2);

    //  X509-     
    X509Certificate cert2 = (X509Certificate) cf
            .generateCertificate(new ByteArrayInputStream(encodedCert2));

    /*   XML-   */

    //    XML-
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,         XML-
    dbf.setIgnoringElementContentWhitespace(true);

    //  ,    CDATA      XML-
    dbf.setCoalescing(true);

    //  ,       XML-
    dbf.setNamespaceAware(true);

    //         
    DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
    Document doc = documentBuilder.parse(testDoc);

    /*     <ds:Signature>   XML- */

    //           34.10-2001
    XMLSignature sig = new XMLSignature(doc, "", signMethod);

    //     
    sig.setId(Id1);

    //    XML-
    Element anElement = doc.getDocumentElement();

    //     XML-   
    anElement.appendChild(sig.getElement());

    /*     XML-         */

    //    <ds:Transforms>  XML-
    Transforms transforms = new Transforms(doc);

    //        
    transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
    transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);
    String[][] filters = {{XPath2FilterContainer.SUBTRACT, "//ds:Signature"}};
    transforms.addTransform(Transforms.TRANSFORM_XPATH2FILTER,
            XPath2FilterContainer.newInstances(doc, filters));

    //       ( <ds:Reference>),    
    // XML- (       <ds:Transforms> 
    //    )
    sig.addDocument("", transforms, digestMethod);

    /*      XML-       ,
        */

    //       <ds:KeyInfo>      
    //    
    sig.addKeyInfo(cert1);

    //    XML-
    sig.sign(privateKey1);

    /*     <ds:Signature>   XML- */

    //           34.10-2001
    XMLSignature sigSecond = new XMLSignature(doc, "", signMethod);

    //     
    sigSecond.setId(Id2);

    //     XML-   
    anElement.appendChild(sigSecond.getElement());

    /*     XML-         */

    //    <ds:Transforms>  XML-
    Transforms transforms2 = new Transforms(doc);

    //        
    transforms2.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
    transforms2.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);
    transforms2.addTransform(Transforms.TRANSFORM_XPATH2FILTER,
            XPath2FilterContainer.newInstances(doc, filters));

    //       ( <ds:Reference>),    
    // XML- (       <ds:Transforms> 
    //    )
    sigSecond.addDocument("", transforms2, digestMethod);

    /*      XML-       ,
        */

    //       <ds:KeyInfo>      
    //    
    sigSecond.addKeyInfo(cert2);

    //    XML-
    sigSecond.sign(privateKey2);

    /*   XML-   */

    //  ,      XML-
    FileOutputStream os = new FileOutputStream(signTwo);

    //     XML-  
    TransformerFactory tf = TransformerFactory.newInstance();

    //     XML-  
    Transformer trans = tf.newTransformer();

    //   XML-  
    trans.transform(new DOMSource(doc), new StreamResult(os));
    os.close();
}

/**
 *      XML-    
 * 34.10-2001.
 *
 * @param signTwo  
 * @throws Exception /
 */
public static void twoSignDocVer(String signTwo) throws Exception {
    //    XML-
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    //  ,         XML-
    dbf.setIgnoringElementContentWhitespace(true);

    //  ,    CDATA      XML-
    dbf.setCoalescing(true);

    //  ,       XML-
    dbf.setNamespaceAware(true);

    //         
    DocumentBuilder documentBuilder = dbf.newDocumentBuilder();

    /*   XML-   */
    Document doc = documentBuilder.parse(signTwo);

    /*    <ds:Signature>  XML- */

    //        Signature
    Element nscontext = doc.createElementNS(null, "namespaceContext");
    nscontext.setAttributeNS("http://www.w3.org/2000/xmlns/",
            "xmlns:" + "ds".trim(), Constants.SignatureSpecNS);

    //    
    Element[] sigElement = new Element[2];

    //          <ds:Signature>
    sigElement[0] = (Element) XPathAPI.selectSingleNode(doc,
            "//ds:Signature[@Id='FirstSignature']", nscontext);

    //          <ds:Signature>
    sigElement[1] = (Element) XPathAPI.selectSingleNode(doc,
            "//ds:Signature[@Id='SecondSignature']", nscontext);

    /*    XML-       ,  
   XML- */

    //     
    XMLSignature[] signature = new XMLSignature[2];

    //     
    signature[0] = new XMLSignature(sigElement[0], "");

    //     
    signature[1] = new XMLSignature(sigElement[1], "");

    //     
    for (int i = 0; i < 2; i++) {

        //   <ds:KeyInfo>    
        KeyInfo ki = signature[i].getKeyInfo();

        //        
        X509Certificate certKey = ki.getX509Certificate();

        //   ,   
        //    
        if (certKey != null) {
            Logger.getLogger("LOG")
                    .info("The XML signature " + (i + 1) + " is " +
                            (signature[i].checkSignatureValue(certKey)
                                    ? "valid (good)" : "invalid (bad)"));
        }
        //        
        else {
            //         
            PublicKey pk = ki.getPublicKey();

            //    ,      
            if (pk != null) {
                Logger.getLogger("LOG")
                        .info("The XML signature " + (i + 1) + " is " +
                                (signature[i].checkSignatureValue(pk)
                                        ? "valid (good)" : "invalid (bad)"));
            }
            //        
            else throw new Exception(
                    "There are no information about public key. Verification couldn't be implemented");

        }
    }

}
}
